-------------------------------------------------------------------------------
-- Project  ELE8304 : Circuits intégrés à très grande échelle
-------------------------------------------------------------------------------
-- File     wb_tb.vhd
-- Authors  Titouan Luard <luardtitouan@gmail.com> 
--          Yann Roberge <yann.roberge@polymtl.ca> 
-- Lab      GRM - Polytechnique Montreal
-- Date     2021-11-19
-------------------------------------------------------------------------------
-- Brief    test bench of the fetche
-------------------------------------------------------------------------------
library ieee;
use ieee.std_logic_1164.all;
use ieee.numeric_std.all;
use ieee.math_real.all;

library work;
use work.riscv_pkg.all;

entity wb_tb is
end wb_tb;

architecture tb of wb_tb is

    constant XLEN   : positive := 32;

    
    signal alu_result    : std_logic_vector(XLEN-1 downto 0);
    signal rd_addr       : std_logic_vector(9 downto 0);
    signal rd_addr_o       : std_logic_vector(9 downto 0);
    signal rd_data      : std_logic_vector(XLEN-1 downto 0);
    signal dmem_read     : std_logic_vector(XLEN-1 downto 0);
    signal opcode:  std_logic_vector(OPCODE_WIDTH-1 downto 0);
        
    signal expected_result_data : std_logic_vector(XLEN-1 downto 0);
    signal expected_result_rd       : std_logic_vector(9 downto 0);
    signal expected_result_rd_data      : std_logic_vector(XLEN-1 downto 0);
    signal expected_result_alu     : std_logic_vector(XLEN-1 downto 0);
    constant PERIOD   : time := 10 ns;

begin

  -- DUT
  dut: entity work.wb

    port map (
      i_alu_result  => alu_result,
      i_rd_addr => rd_addr,
      i_dmem_read =>dmem_read,
      i_opcode => opcode,
      o_rd_data => rd_data,
      o_rd_addr => rd_addr_o
       );

  -- Main TB process
  p_main : process
  begin
    -- Vecteurs de tests

    report "BEGIN SIMULATION";
    --t=0 ns

    report "initialisation";


    alu_result <= (0 downto 0 => '1', others => '0') ;
    rd_addr <= (0 downto 0 => '1', others => '0');
    dmem_read <= ( others => '0');
    opcode <= "0000000";
    
    expected_result_alu <= (0 downto 0 => '1', others => '0') ;
    expected_result_rd <= (0 downto 0 => '1', others => '0'); 
    expected_result_rd_data   <= (0 downto 0 => '1', others => '0') ;

  wait for PERIOD; --10 ns 

    assert rd_addr = expected_result_rd
      report "rd_addr error" severity error;  
    assert rd_data = expected_result_rd_data
      report "rd_data error" severity error;  
  opcode <= OPCODE_LW;
  
  expected_result_rd_data   <= (others => '0') ;
    
    wait for PERIOD; --20 ns
    assert rd_addr = expected_result_rd
      report "rd_addr error" severity error;  
    assert rd_data = expected_result_rd_data
      report "rd_data error" severity error;  

    report "SIMULATION DONE";
    wait;
  end process p_main;

end architecture tb;

